! ---------------------- BEGIN SKIP -------------------
      subroutine SKIP(lunit, nlines)
        if (nlines .lt. 1) then
          return
        else
          do i = 1, nlines
             read(lunit, *)
          end do
          return
        end if
      end
! ---------------------- END SKIP -------------------
! ------------------------------------------------------------------ skipcmnt
      subroutine skipcmnt(nu, comment, ncomments)

! Skip text comments in the file attached to unit nu, but save skipped 
! comments in character array comment.  Skip at least one line, and more as 
! long as the lines are preceded by "|" or "!".

! Dates: 04/16/01 - Written by D. Boore
!        12/07/01 - Added check for eof
!        11/04/03 - Use trim_c to trim possible leading blank
!        02/03/07 - Initialize comments to blank
!        04/28/15 - Replaced comment characters * or C with ! (The Fortran 95 standard)

      character comment(*)*(*), buf*80

      ncomments = 0
100   buf = ' '
      read (nu,'(a)',end=999) buf
      call trim_c(buf,nc_buf)
      if (buf(1:1) .eq.'!' .or. buf(1:1) .eq.'|' .or. 
     :                     ncomments + 1 .eq. 1) then
        ncomments = ncomments + 1
        comment(ncomments) = ' '
        comment(ncomments) = buf(1:nc_buf)
        goto 100
      else 
        backspace nu
      end if

999   continue
 
      return
      end
! ------------------------------------------------------------------ skipcmnt

! --------------------------- BEGIN GET_LUN ----------------
      subroutine get_lun(lun)

! Finds a logical unit number not in use; returns
! -1 if it cannot find one.

! Dates -- 05/19/98 - Written by D. Boore, following
!                     Larry Baker's suggestion
!        04/28/15 - Replaced comment characters * or C with ! (The Fortran 95 standard)

      logical isopen
      do i = 99,10,-1
        inquire (unit=i, opened=isopen)
        if(.not.isopen) then
          lun = i
          return
        end if
      end do
      lun = -1

      return
      end
! --------------------------- END GET_LUN ----------------
     
! --------------------- BEGIN UPSTR ----------------------------------
      Subroutine UPSTR ( text )
! Converts character string in TEXT to uppercase
! Dates: 03/12/96 - Written by Larry Baker
!        04/28/15 - Replaced comment characters * or C with ! (The Fortran 95 standard)

!
      Implicit   None
!
      Character  text*(*)
!
      Integer    j
      Character  ch
!
      Do 1000 j = 1,LEN(text)
         ch = text(j:j)
         If ( LGE(ch,'a') .and. LLE(ch,'z') ) Then
            text(j:j) = CHAR ( ICHAR(ch) - ICHAR('a') + ICHAR('A') )
         End If
 1000    Continue
!
      Return
      End
! --------------------- END UPSTR ----------------------------------
! --------------------------- BEGIN TRIM_C -----------------------
      subroutine trim_c(cstr, nchar)

! strips leading and trailing blanks from cstr, returning the
! result in cstr, which is now nchar characters in length

! Strip off tabs also.

! Here is a sample use in constructing a column header, filled out with 
! periods:

!* Read idtag:
!        idtag = ' '
!        read(nu_in, '(1x,a)') idtag
!        call trim_c(idtag, nc_id)
!* Set up the column headings:
!        colhead = ' '
!        colhead = idtag(1:nc_id)//'......' ! nc_id + 6 > length of colhead

! Dates: 12/23/97 - written by D. Boore
!        12/08/00 - pad with trailing blanks.  Otherwise some comparisons
!                   of the trimmed character string with other strings
!                   can be in error because the original characters are left
!                   behind after shifting.  For example, here is a string
!                   before and after shifting, using the old version:
!                      col:12345
!                           MTWH  before
!                          MTWHH  after (but nc = 4).
!        03/21/01 - Check for a zero length input string
!        11/09/01 - Change check for zero length string to check for all blanks
!        10/19/09 - Strip off tabs
!        04/28/15 - Replaced comment characters * or C with ! (The Fortran 95 standard)

      character cstr*(*)

      if(cstr .eq. ' ') then
        nchar = 0
        return
      end if

      nend = len(cstr)

! Replace tabs with blanks:

      do i = 1, nend
        if(ichar(cstr(i:i)) .eq. 9) then
           cstr(i:i) = ' '
        end if
      end do



!      if(nend .eq. 0) then
!        nchar = 0
!        return
!      end if

      do i = nend, 1, -1
        if (cstr(i:i) .ne. ' ') then
           nchar2 = i
           goto 10
        end if
      end do

10    continue

      do j = 1, nchar2
        if (cstr(j:j) .ne. ' ') then
          nchar1 = j
          goto 20
        end if
      end do

20    continue
   
      nchar = nchar2 - nchar1 + 1
      cstr(1:nchar) = cstr(nchar1: nchar2)
      if (nchar .lt. nend) then
        do i = nchar+1, nend
          cstr(i:i) = ' '
        end do
      end if

      return
      end
! --------------------------- END TRIM_C -----------------------


! ------------------------------------------------------------- SMC_Npts
      subroutine smc_npts(f_smc, npts, sps)

* Open smc file and obtain length of time series:

* Dates: 12/07/00 - Written by D. Boore
*        07/17/03 - Read all of int_head and real_head, and add sps to output
!        04/04/10 - Add trim_c, and use nc_f_smc.

      integer int_head(48)
      real real_head(50)
      character f_smc*(*)

      call get_lun(nu_smc)
      call trim_c(f_smc,nc_f_smc)
      open(unit=nu_smc,file=f_smc(1:nc_f_smc),status='unknown')
      call skip(nu_smc,11)

      read(nu_smc, '(8I10)') (int_head(i), i=1, 48)
      npts = int_head(17)

      read(nu_smc, '(5e15.7)') (real_head(i), i = 1, 50)
      sps = real_head(2)

      close(nu_smc)        

      return
      end
! ------------------------------------------------------------- SMC_Npts  

!------------------   Begin SMCRead   ----------------------------------
      subroutine SMCRead(f_in,  
     :                   tskip, tlength,
     :                   y, char_head, int_head, real_head, comments)

!  If tskip = 0.0, no data are skipped
!  If tlength = 0.0, only headers are read
!  If tlength < 0.0, read all data, less tskip
!  Otherwise tlength of data is read, after skipping tskip, unless 
!  tskip + tlength exceeds the length of the input time series, in which
!  case tlength is adjusted so that all data, less tskip, are read.  Thus
!  setting tlength to a large number (or setting it to a negative number)
!  guarantees that all data (minus tskip) are read.

! The program requires get_lun.for

! The output arrays char_head, int_head, real_head should be dimensioned
! as follows in the calling program:

!      character char_head(11)*80, comments(500)*80 ! "80" is the critical parameter
!      integer int_head(48)
!      real real_head(50)

!  Dates: 07/17/97 - Modified order of input parameters, to put input
!                    first (unit, file, tskip), followed by output.
!         12/18/97 - Read tlength
!         05/19/98 - Determine unit from within program; remove "unit"
!                    from argument list
!         03/09/99 - Change logic to determine amount of record to
!                    read and allow reading of all record, minus tskip,
!                    by setting tlength < 0.0.
!         05/18/99 - Deleted npts, sps from input parameter list.  Added 
!                    character array with comments string (allow for 
!                    50 comments).
!         12/16/99 - Renamed from ReadSMC
!         05/10/00 - Replaced
!                        tlength = float(npts_out/sps)
!                    with
!                        tlength = float(npts_out)/sps
!                    (the error and correction was pointed out by Wang Guoquan
!                    in email on 05/06/2000)
!         01/04/01 - Check for unevenly spaced data (sps = rnull) and
!                    return 0.0 in y
!         02/07/01 - If tlength = 0, only read headers
!         12/14/01 - Fixed logic for case when tlength < 0 and tskip > 0.
!                    Also, program used to reset tlength, but it does not do
!                    that now.  The user can compute tlength as 
!                    float(int_head(17))/real_head(2)
!         01/01/05 - Use "int" in calculation of npts (do not use
!                    implicit conversion from real to integer)
!         10/15/07 - Use higher precision input format if int_head(47) = 8
!         04/04/10 - Add call to trim_c, and use nc_f_in.
!         10/20/15 - Changed format of high precision input.
!                  - This was not necessary, as the old format statement can read
!                    either e14.7 or e14.7e1 (the new format being used in smcwrite for
!                    high precision).
!         12/30/15 - Replaced "int" with "nint" when computing nskip and npts2read
!                  - A few modernizations of syntax.

      real real_head(*)
      integer int_head(*)
      character*80 char_head(*), comments(*)
      character f_in*(*) 
      real y(*)

      
      
      call get_lun(nu)
      call trim_c(f_in, nc_f_in)
      open(unit= nu, file=f_in(1:nc_f_in), status='unknown')

      do i = 1, 11
        read(nu, '(a)') char_head(i)
      end do

      read(nu, '(8I10)') (int_head(i), i=1, 48)

      read(nu, '(5e15.7)') (real_head(i), i = 1, 50)

      sps = real_head(2)

      do i = 1, int_head(16)
        read(nu,'(a)') comments(i)
      end do

      if(real_head(2) == real_head(1)) then   ! unevenly sampled data

         int_head(17) = 2
         do i = 1, int_head(17)
           y(i) = 0.0
         end do

      else

! Skip into the trace:

        nskip = nint(tskip * sps)
!        nskip = int(tskip * sps)

! How much record to read?

        npts_in = int_head(17)
        npts2read = nint(tlength * sps)
!        npts2read = int(tlength * sps)

        if (tlength < 0.0) then  
          npts_out = npts_in - nskip
        else if (nskip + npts2read <= npts_in) then
          npts_out = npts2read
        else
          npts_out = npts_in - nskip
!          tlength = float(npts_out)/sps
        end if

        int_head(17) = npts_out
      
        if (tlength == 0.0) then
          return    ! only read headers
        else
          if (int_head(47) == 8) then
            read(nu, '(5(e14.7))') (y(i), i = 1, npts_out + nskip)
!            read(nu, '(5(e14.7e1))') (y(i), i = 1, npts_out + nskip)
          else
            read(nu, '(8(1pe10.4e1))') (y(i), i = 1, npts_out + nskip)
          end if
          do i = 1, npts_out
            y(i) = y( i + nskip)
          end do
        end if

      end if

      close(unit=nu)

      return
      end
!------------------   End SMCRead   ----------------------------------





!------------------   Begin SMCWrite  ----------------------------------
      subroutine SMCWrite(f_out, precision,
     :                    y, char_head, itype, 
     :                    int_head, real_head, comments)

!  Reformats a time series 
!  into SMC format (the format used on the CD-ROM)

! If itype < 0, the first text header is given by char_head(1) and
!               not set to one of the standard headers
!    itype = 0 for 'UNKNOWN'
!    itype = 1 for 'UNCORRECTED ACCELEROGRAM'
!    itype = 2 for 'CORRECTED ACCELEROGRAM'
!    itype = 3 for 'VELOCITY'
!    itype = 4 for 'DISPLACEMENT'
!    itype = 5 for 'RESPONSE SPECTRA'

! The program requires:
!        get_lun.for
!        mnmaxidx.for

! The arrays char_head, int_head, real_head must be dimensioned
! by the calling program as follows:
!
!      character char_head(11)*80, comments(N)*80 ! see "N" below
!      integer int_head(48)
!      real real_head(50)

! Note that 
!      int_head(17) = npts
!      real_head(2) = sps
!      int_head(16) = ncomments (dimension array "comments" accordingly
!                                so that N is a number > = ncomments)
!      I have found that if no comments are included, BAP cannot process
!      the resulting file.  For this reason, be sure that at least one comment
!      is included (ncomments = 1, comments(1) = '|').
!
! default values for int_head = -32768
!                    real_head = 1.7e+38

! The minimum required input for char_head is to set each of the 11 variables
! to "*" as in:
!  do i = 1, 11
!    char_head(i) = '*'
!  end do


! Dates: 05/13/99 - revised by D. Boore
!        05/18/99 - continued, adding the comments string array
!        12/15/99 - Renamed from WriteSMC to SMCWrite, and added
!                   itype to argument list and find and write peak
!                   accelerations (itype < 0 to echo input char_head(1)).
!        01/29/00 - Added comments regarding dimensioning of input variables
!        02/18/00 - Substitute 0.0 if abs(y) < 1.0e-09 (otherwise, the field
!                   is stored as ********, because an exponent of -10 is
!                   larger than allowed by the format statement).
!        02/25/00 - Changed output format of peak motion in text headers from
!                   1pe10.3 to 1pe9.3e1
!        02/27/01 - Set char_head(7)(42:60) to blanks before filling with
!                   peak motion
!        08/22/01 - Added comment about minimum requirement for char_head
!        10/15/07 - Use high precision format if precision=HIGH
!        04/24/09 - Changed format of read header output from 5e15.7 to 1p5e15.7
!        04/04/10 - Add call to trim_c, and use nc_f_out.
!        02/02/12 - Compute r_ep if eq and sta coords /= null
!        06/24/13 - Replaced ".eq." with "==", etc.
!                 - Replaced 1pex.x with esx.x
!                 - In writing real_head, revert to the format 5e15.7 rather than 5es15.7,
!                   as the latter might imply more precision than available in the
!                   single precision numbers.
!        10/20/15 - Change high precision output format to include blanks between
!                   numbers (as suggested by Chris Stephens)
!        10/24/00 - For both output precision options substitute 0.0 if abs(y) < 1.0e-09 (otherwise, the field
!                   is stored as ********, because an exponent of -10 is
!                   larger than allowed by the format statement).

      real real_head(*)
      integer int_head(*)
      character*80 char_head(*), comments(*)
      character f_out*(*), chead1(6)*26, precision*(*)
      real y(*)
      data chead1/
     : '0 UNKNOWN                 ', '1 UNCORRECTED ACCELEROGRAM',
     : '2 CORRECTED ACCELEROGRAM  ', '3 VELOCITY                ',
     : '4 DISPLACEMENT            ', '5 RESPONSE SPECTRA        '/

! Open output file:
      call get_lun(nu_out)
      call trim_c(f_out, nc_f_out)
      open(unit=nu_out, file=f_out(1:nc_f_out), status='unknown')

! Write name to screen:
      print *, 
     :  ' nu_out, Output file name = ', nu_out, ', ', f_out(1:nc_f_out)

! Get peak motions:
      
      call mnmaxidx(y, 1, int_head(17), 1,
     :             ymin, ymax, indx_min, indx_max) 
      peak_mtn = ymax
     
      if (abs(ymin) > abs(ymax)) peak_mtn = ymin
      
! Compute r_ep if eq, sta coords /= null:  NOTE: Need to include distaz in the calling program

      rnull = real_head(1)
      if (real_head(3) /= rnull .and. real_head(3) /= rnull) then  ! eq coords not null
        if (real_head(11) /= rnull .and. real_head(12) /= rnull) then  ! sta coords not null
          eqlat = real_head(3)
          eqlong = real_head(4)
          stalat = real_head(11)
          stalong = real_head(12)
          call distaz( -1.0, eqlat,eqlong,stalat,stalong,
     :    rdeg, rkm, az, baz)
          real_head(17) = rkm
        end if
      end if
          

! Write headers:

! First write the 11 lines of comments:

      if (itype < 0) then
        write(nu_out,'(a)') char_head(1)
      else
        write(nu_out,'(a)') chead1(itype+1)
      end if

      char_head(7)(34:41) = 'pk mtn ='    ! makes more sense than 'pk acc ='
      char_head(7)(42:60) = ' '
      write(char_head(7)(42:50),'(1pe9.3e1)') peak_mtn

      do i = 2, 11
        write(nu_out,'(a)') char_head(i)
      end do
          
      if (precision(1:4) == 'HIGH') then
        int_head(47) = 8
      else
        int_head(47) = int_head(1) ! null value
      end if
      
      write(nu_out, '(8I10)') (int_head(i), i=1, 48)

      dt = 1.0/real_head(2)
      real_head(29) = float(indx_max - 1) * dt
      real_head(30) = ymax
      real_head(31) = float(indx_min - 1) * dt
      real_head(32) = ymin

!      write(nu_out, '(5es15.7)') (real_head(i), i = 1, 50)
!DEBUG
!      write(nu_out, '(a, 1x,e15.7, 1x,f7.2, 1x,es12.5)') 
!     :   ' In SMCWRITE: real_head(2), dt = ', 
!     :     real_head(2), real_head(2), dt
!DEBUG
      write(nu_out, '(5e15.7)') (real_head(i), i = 1, 50)


      if (int_head(16) > 0) then
        do i = 1, int_head(16)
          write(nu_out,'(a)') comments(i)
        end do
      end if
          
        do i = 1, int_head(17)
          if (abs(y(i)) < 1.0e-09) y(i) = 0.0
        end do

      if (precision(1:4) == 'HIGH') then
        write(nu_out, '(5(es14.7e1))') (y(i), i = 1, int_head(17))
      else
!        do i = 1, int_head(17)
!          if (abs(y(i)) < 1.0e-09) y(i) = 0.0
!        end do
        write(nu_out, '(8(es10.4e1))') (y(i), i = 1, int_head(17))
      end if
      

! That should be it; close the output file.

      close(unit=nu_out)

      return
      end
!------------------   End SMCWrite   ----------------------------------


! ------------------------ BEGIN DISTAZ ----------------------
      subroutine distaz( wlongsign, alat, along, blat, blong,
     * rdeg, rkm, az, baz)
! 
! compute distances, azimuths using formulas from
! Bruce Julian.
!
! latest modification: 1/27/84
!
!        11/24/07 - Trap for colocated pair in subroutine distaz
!        05/01/15 - Replaced comment characters * or C with ! (The Fortran 95 standard)

      if (alat .eq. blat .and. along .eq. blong) then
        rdeg = 0.0
        rkm = 0.0
        az = 0.0
        baz = 0.0
        return
      end if

      pi = 4.0 * atan( 1. )
      dtor = pi/ 180.
!
! convert from degrees to radians and correct sign of
! longitude so that east longitude is positive.
!
      alatr = dtor * alat
      alongr = -dtor * along * wlongsign
      blatr = dtor * blat
      blongr = -dtor * blong * wlongsign
!
! compute geocentric latitudes.
!
      alatr = atan( 0.993305 * tan( alatr ) )
      blatr = atan( 0.993305 * tan( blatr ) )
!
! compute latitude dependent quantities
!
      ca = cos( alatr )
      cb = cos( blatr )
      sa = sin( alatr )
      sb = sin( blatr )
!
! now compute other quantities
!
      a = cb * sin( blongr - alongr )
      b = ca * sb - sa * cb * cos( blongr - alongr )
      cd = ca * cb * cos( blongr - alongr ) + sa * sb
      sd = sqrt( a*a + b*b )
!
! compute distances
!
      rdeg = atan2( sd, cd )/ dtor
      rkm = 111.19 * rdeg
!
! compute azimuth (from a to b) and make it positive.
!
      az = atan2( a, b )/ dtor
      if ( az .lt. 0.0 ) az = az + 360.0
!
! compute back azimuth (from b to a) and make it positive.
!
      a = ca * sin( alongr - blongr )
      b = cb * sa - sb * ca * cos( alongr - blongr )
      baz = atan2( a, b)/ dtor
      if ( baz .lt. 0.0 ) baz = baz + 360.0
!
      return
      end
! ------------------------ END DISTAZ ----------------------


! ---------------------- BEGIN MNMAXIDX ----------------------
      subroutine mnmaxidx(a,nstrt,nstop,ninc,
     :                    amin,amax,indx_min,indx_max) 

! A rewrite of mnmax, returning the indices of the min and max values
!
! Dates: 10/05/99 - written by D. M. Boore
!        05/16/02 - Replaced "dimension a(1)" with "real a(*)"
!        11/03/12 - Replace if-then with if--then-else

      real a(*)
      amax = a( nstrt)
      indx_max = nstrt
      amin=amax         
      indx_min = nstrt
      do i=nstrt,nstop,ninc
      
        if (a(i) > amax) then
          amax=a(i)
          indx_max = i
        else if (a(i) < amin) then
          amin=a(i)
          indx_min = i
        end if
        
      end do
      
      return                  
      end                     
! ---------------------- END MNMAXIDX ----------------------


! ---------------------- BEGIN IMNMAX ----------------------
      subroutine imnmax(ia,nstrt,nstop,ninc,imin,imax) 
c
* Compute min, max for an integer array

c author: D. M. Boore
c last change: 7/17/97 - Written, based on mnmax.for
c
      integer ia(*)
      imax = ia( nstrt)
      imin=imax         
      do 10 i=nstrt,nstop,ninc
      if(ia(i)-imax) 15,15,20  
20    imax=ia(i)               
      go to 10                
15    if(ia(i)-imin) 25,10,10  
25    imin=ia(i)               
10    continue                
      return                  
      end                     
! ---------------------- END IMNMAX ----------------------

!---------------------------------------------------------------- datetime
      subroutine datetime(date_c, time_c, iflag)

! Returns character strings with date and time; the routines
! that provide date and time are system dependent, and iflag can be used
! to specify which routines are to be called.

! Dates: 02/16/01 - Written by D. Boore, modifying \smsim\datetime.for
!        06/15/01 - Removed specification of integers iyr, imon, etc.
!        04/29/15 - Replaced comment characters * or C with ! (The Fortran 95 standard)

! the character string for date
      character date_c*(*), time_c*(*)

      if (iflag .eq. 0) then
! Lahey compiler version:
        call DATE(date_c)   ! character date_c*8, but *10 is OK
        call TIME(time_c)   ! character time_c*11
      else
! Jump to this block if needed
      end if

      return
      end
!---------------------------------------------------------------- datetime


! ----------------------------- BEGIN RMVTREND ----------------
      subroutine rmvtrend(y, n)

! Removes a straightline fit to first and last points, replacing
! the input array with the detrended array

! Dates: 02/09/99 - written by D. Boore
!        04/28/15 - Replaced comment characters * or C with ! (The Fortran 95 standard)


      real y(*)

      y1 = y(1)
      y2 = y(n)
      slope = (y2 - y1)/float(n-1)

      do i = 1, n
        y(i) = y(i) - (y1 + slope*float(i-1))
      end do

      return
      end
! ----------------------------- END RMVTREND ----------------


! ----------------------- BEGIN RCC -----------------------------------
      subroutine RCC (KPARM,CSTR_IN,NCSTR_IN,C,NC,ibgn,iend)
! RCC - [MUELLER.FS.GEN]CSMGENLB
! Read character string C(1:NC), the KPARMth element in CSTR_IN(1:NCSTR_IN).
!  CSTR_IN contains one-or-more mixed-type fields
!(OLD COMMENT:c  separated by blanks or commas to the right of an optional '='.)
!  KPARMth field contains C to the right of an optional '>'.
! If the KPARMth field is nonexistent or empty, RCC sets NC=0 and returns.
!  This simulates a <carriage-return> in a Q-format read.
!  (An empty field must be separated by commas, not blanks.)

! Dates:  xx/xx/xx - Written by C. Mueller, USGS
!         06/22/98 - Modified by L. Baker
!         11/12/00 - Dave Boore introduced a working array, and a call to 
!                    rmv_tabs (this replaces a tab with a blank, but does not
!                    expand the tabs).  This was done because a file in which 
!                    fields are separated by tabs characters will not be parsed 
!                    correctly; apparently the tab characters (ascii 9) is 
!                    not the same thing as a blank.
!         12/04/00 - added ibgn, iend to rcf, rci, rcc calling arguments
!         06/11/01 - Disable use of "=" to indicate where the parameters
!                    start.  I did this because I sometimes add comments
!                    to the right of the parameters, and if these comments
!                    contain the character "=" then the parameters are not
!                    parsed correctly.  I reasoned that eliminating the check
!                    for "=" is the best solution to the problem (as opposed
!                    to restricting the use of "=" to only indicate 
!                    where the parameters start) because I rarely used
!                    "=" in this way.
!         06/12/01 - Minor modification (i2 = 0 rather than i2 = 1)
!         10/29/02 - Changed "rmv_tabs" to "tabs_rmv"
!         07/25/10 - Increase dimension of cstr
!        05/01/15 - Replaced comment characters * or C with ! (The Fortran 95 standard)

      character CSTR_IN*(*),C*(*),term*1

      character cstr*500

      if (ncstr_in .gt. 500) then
         write(*,'(a, 1x,i5, a)') '  WARNING: IN RCC, NCSTR_IN = ', 
     :          ncstr_in,
     :         ' WHICH IS GREATER THAN THE DIMENSION OF THE'//
     :         ' WORK ARRAY; QUITTING!!!!!'
         return
      end if

      ncstr = ncstr_in

      cstr = ' '
      cstr(1: ncstr) = cstr_in
      call tabs_rmv(cstr, ncstr)

! Eliminate trailing white space.
      N = NCSTR+1
1     N = N-1
      if (CSTR(N:N).eq.' '.or.CSTR(N:N).eq.',') goto 1

! Changes on 6/11/01:
!c Find optional '='.
!      I2 = INDEX(CSTR(1:N),'=')
!      term = '='
      i2 = 0         ! DMB, 6/12/01
      term = ' '     ! DMB, 6/11/01
! Changes on 6/11/01:

! Find start and end of KPARMth field.
      do 4 K=1,KPARM
         I1 = I2
2        I2 = I1
3        I1 = I1+1
         if (CSTR(I1:I1).eq.' ') goto 3
         if (term.eq.' '.and.CSTR(I1:I1).eq.',') then
            term = ','
            goto 2
         end if
         IBLANK = INDEX(CSTR(I1:N),' ')
         ICOMMA = INDEX(CSTR(I1:N),',')
         if (IBLANK.eq.0.and.ICOMMA.eq.0.and.K.lt.KPARM) goto 801
         if (IBLANK.eq.0) IBLANK = N-I1+2
         if (ICOMMA.eq.0) ICOMMA = N-I1+2
         I2 = I1+MIN(IBLANK,ICOMMA)-1
         if (I2.gt.N) then
            term = '.'
         else
            term = CSTR(I2:I2)
         end if
4        continue
! Check for empty field.
      if (I2.eq.I1) goto 801
! Find optional '>'.
      I1 = I1+INDEX(CSTR(I1:I2),'>')
! Get C.
      C = CSTR(I1:I2-1)
      NC = I2-I1
      ibgn = i1
      iend = i2-1
      return
! Errors.
801   NC = 0
      return
      end
! ----------------------- END RCC -----------------------------------


      
! --------------------------- BEGIN RCF ------------------------------
      subroutine RCF (KPARM,CSTR_IN,NCSTR_IN,F,ibgn,iend, ISTAT)
! RCF - [MUELLER.FS.GEN]CSMGENLB
! Read real F, the KPARMth element in CSTR_IN(1:NCSTR_IN).
!  CSTR_IN contains one-or-more mixed-type fields
!(OLD COMMENT:c  separated by blanks or commas to the right of an optional '='.)
!  KPARMth field contains F to the right of an optional '>'.
! ISTAT= 0 for success;
!      = 1 if CSTR contains fewer than KPARM fields;
!      = 2 if KPARMth field is empty;
!      = 3 if KPARMth field is unreadable;
!      if ISTAT>0, return with F unchanged.


! Dates:  xx/xx/xx - Written by C. Mueller, USGS
!         06/22/98 - Modified by L. Baker
!         11/12/00 - Dave Boore introduced a working array, and a call to 
!                    rmv_tabs (this replaces a tab with a blank, but does not
!                    expand the tabs).  This was done because a file in which 
!                    fields are separated by tabs characters will not be parsed 
!                    correctly; apparently the tab characters (ascii 9) is 
!                    not the same thing as a blank.
!         12/04/00 - added ibgn, iend to rcf, rci, rcc calling arguments
!         06/11/01 - Disable use of "=" to indicate where the parameters
!                    start.  I did this because I sometimes add comments
!                    to the right of the parameters, and if these comments
!                    contain the character "=" then the parameters are not
!                    parsed correctly.  I reasoned that eliminating the check
!                    for "=" is the best solution to the problem (as opposed
!                    to restricting the use of "=" to only indicate 
!                    where the parameters start) because I rarely used
!                    "=" in this way.
!         06/12/01 - Minor modification (i2 = 0 rather than i2 = 1)
!         10/29/02 - Changed "rmv_tabs" to "tabs_rmv"
!         07/25/10 - Increase dimension of cstr
!        05/01/15 - Replaced comment characters * or C with ! (The Fortran 95 standard)

      character CSTR_IN*(*), term*1

      character cstr*500

      if (ncstr_in .gt. 500) then
         write(*,'(a, 1x,i5, a)') '  WARNING: IN RCF, NCSTR_IN = ', 
     :          ncstr_in,
     :         ' WHICH IS GREATER THAN THE DIMENSION OF THE'//
     :         ' WORK ARRAY; QUITTING!!!!!'
         return
      end if

      ncstr = ncstr_in

      cstr = ' '
      cstr(1: ncstr) = cstr_in
      call tabs_rmv(cstr, ncstr)

      ISTAT = 0
! Eliminate trailing white space.
      N = NCSTR+1
1     N = N-1
      if (CSTR(N:N).eq.' '.or.CSTR(N:N).eq.',') goto 1

! Changes on 6/11/01:
!c Find optional '='.
!      I2 = INDEX(CSTR(1:N),'=')
!      term = '='
      i2 = 0         ! DMB, 6/12/01
      term = ' '     ! DMB, 6/11/01
! Changes on 6/11/01:

! Find start and end of KPARMth field.
      do 4 K=1,KPARM
         I1 = I2
2        I2 = I1
3        I1 = I1+1
         if (CSTR(I1:I1).eq.' ') goto 3
         if (term.eq.' '.and.CSTR(I1:I1).eq.',') then
            term = ','
            goto 2
         end if
         IBLANK = INDEX(CSTR(I1:N),' ')
         ICOMMA = INDEX(CSTR(I1:N),',')
         if (IBLANK.eq.0.and.ICOMMA.eq.0.and.K.lt.KPARM) goto 801
         if (IBLANK.eq.0) IBLANK = N-I1+2
         if (ICOMMA.eq.0) ICOMMA = N-I1+2
         I2 = I1+MIN(IBLANK,ICOMMA)-1
         if (I2.gt.N) then
            term = '.'
         else
            term = CSTR(I2:I2)
         end if
4        continue
! Check for empty field.
      if (I2.eq.I1) goto 802
! Find optional '>'.
      I1 = I1+INDEX(CSTR(I1:I2),'>')
! Get F.
10    read (CSTR(I1:I2-1),90010,err=803) F
90010 format (f15.0)
      ibgn = i1
      iend = i2 - 1
      return
! Errors.
801   ISTAT = 1
      return
802   ISTAT = 2
      return
803   ISTAT = 3
      return
      end
! --------------------------- END RCF ------------------------------



! -------------------------- BEGIN RCI -----------------------
      subroutine RCI (KPARM,CSTR_IN,NCSTR_IN,I,ibgn,iend,ISTAT)
! RCI - [MUELLER.FS.GEN]CSMGENLB
! Read integer I, the KPARMth element in CSTR_IN(1:NCSTR_IN).
!  CSTR_IN contains one-or-more mixed-type fields
!(OLD COMMENT:c  separated by blanks or commas to the right of an optional '='.)
!  KPARMth element contains I to the right of an optional '>'.
! ISTAT= 0 for success;
!      = 1 if CSTR contains fewer than KPARM fields;
!      = 2 if KPARMth field is empty;
!      = 3 if KPARMth field is unreadable;
!      if ISTAT>0, return with I unchanged.

! Dates:  xx/xx/xx - Written by C. Mueller, USGS
!         06/22/98 - Modified by L. Baker
!         11/12/00 - Dave Boore introduced a working array, and a call to 
!                    rmv_tabs (this replaces a tab with a blank, but does not
!                    expand the tabs).  This was done because a file in which 
!                    fields are separated by tabs characters will not be parsed 
!                    correctly; apparently the tab characters (ascii 9) is 
!                    not the same thing as a blank.
!         12/04/00 - added ibgn, iend to rcf, rci, rcc calling arguments
!         06/11/01 - Disable use of "=" to indicate where the parameters
!                    start.  I did this because I sometimes add comments
!                    to the right of the parameters, and if these comments
!                    contain the character "=" then the parameters are not
!                    parsed correctly.  I reasoned that eliminating the check
!                    for "=" is the best solution to the problem (as opposed
!                    to restricting the use of "=" to only indicate 
!                    where the parameters start) because I rarely used
!                    "=" in this way.
!         06/12/01 - Minor modification (i2 = 0 rather than i2 = 1)
!         10/29/02 - Changed "rmv_tabs" to "tabs_rmv"
!         07/25/10 - Increase dimension of cstr
!        05/01/15 - Replaced comment characters * or C with ! (The Fortran 95 standard)

      character CSTR_IN*(*),term*1

      character cstr*500

      if (ncstr_in .gt. 500) then
         write(*,'(a, 1x,i5, a)') '  WARNING: IN RCI, NCSTR_IN = ', 
     :          ncstr_in,
     :         ' WHICH IS GREATER THAN THE DIMENSION OF THE'//
     :         ' WORK ARRAY; QUITTING!!!!!'
         return
      end if

      ncstr = ncstr_in

      cstr = ' '
      cstr(1: ncstr) = cstr_in
      call tabs_rmv(cstr, ncstr)

      ISTAT = 0
! Eliminate trailing white space.
      N = NCSTR+1
1     N = N-1
      if (CSTR(N:N).eq.' '.or.CSTR(N:N).eq.',') goto 1

! Changes on 6/11/01:
!c Find optional '='.
!      I2 = INDEX(CSTR(1:N),'=')
!      term = '='
      i2 = 0         ! DMB, 6/12/01
      term = ' '     ! DMB, 6/11/01
! Changes on 6/11/01:

! Find start and end of KPARMth field.
      do 4 K=1,KPARM
         I1 = I2
2        I2 = I1
3        I1 = I1+1
         if (CSTR(I1:I1).eq.' ') goto 3
         if (term.eq.' '.and.CSTR(I1:I1).eq.',') then
            term = ','
            goto 2
         end if
         IBLANK = INDEX(CSTR(I1:N),' ')
         ICOMMA = INDEX(CSTR(I1:N),',')
         if (IBLANK.eq.0.and.ICOMMA.eq.0.and.K.lt.KPARM) goto 801
         if (IBLANK.eq.0) IBLANK = N-I1+2
         if (ICOMMA.eq.0) ICOMMA = N-I1+2
         I2 = I1+MIN(IBLANK,ICOMMA)-1
         if (I2.gt.N) then
            term = '.'
         else
            term = CSTR(I2:I2)
         end if
4        continue
! Check for empty field.
      if (I2.eq.I1) goto 802
! Find optional '>'.
      I1 = I1+INDEX(CSTR(I1:I2),'>')
! Get I.
10    read (CSTR(I1:I2-1),90010,err=803) I
90010 format (i12)
      ibgn = i1
      iend = i2-1
      return
! Errors.
801   ISTAT = 1
      return
802   ISTAT = 2
      return
803   ISTAT = 3
      return
      end
! -------------------------- END RCI -----------------------



! ---------------------------------------------------------------- Tabs_Rmv
      subroutine tabs_rmv(string, nchar)

! Replaces tabs in the character string with a blank

! Dates: 11/04/00 - Written by D. Boore
!        10/29/02 - Renamed from rmv_tabs.for
!        05/01/15 - Replaced comment characters * or C with ! (The Fortran 95 standard)

      character string*(*)


      do i = 1, nchar
        if(ichar(string(i:i)) .eq. 9) then
           string(i:i) = ' '
        end if
      end do

      return
      end
! ---------------------------------------------------------------- Tabs_Rmv

! -------------------------------------------------- hicut_fd_cosine_taper
      subroutine hicut_fd_cosine_taper(
     : ts_in, sps_in, npts_in, 
     : dc_remove, taper_front, taper_back, fcut1, fcut2, eta,
     : ts_out, sps_out, npts_out,
     : t_in_fft, data_length, df, fnyq_in, fupper_in_fft, n1, n2, npw2)
     
!INPUT:  ts_in, sps_in, npts_in, fcut1, fcut2,
!        dc_remove, taper_front, taper_back
 
!OUPUT:  ts_out, sps_out, npts_out

 
! Dates: 09/16/12 - Written by D.M. Boore, patterned after 
!                   interpolate_time_series_using_fft.for
!        07/01/13 - Generalize response to allow the raised cosine to be raised to a power (this
!                   fit better FAS of some data).
!        12/05/16 - Add t_in_fft, data_length, df, fnyq_in, fupper_in_fft, n1, n2, npw2 to output
!        01/03/17 - Changed include statement from
!                       include '\forprogs\hicut_fd_cosine_taper_subroutines.for'
!                   to
!                       include '\forprogs\hicut_fd_cosine_taper_fft_subroutines.for'

 
!
! Dimension and declaration statements:

      real ts_in(*), ts_out(*) 
      real work(:)
      allocatable :: work

      real fcut1, fcut2, eta
      integer npts_in, npts_out, npw2
      real tlength, ftaper, signnpw2
      real sps_in, sps_out
 
      logical dc_remove, detrend

      signnpw2 = +1.0
      call get_npw2(npts_in,signnpw2,npw2)
      
      allocate( work(npw2) )

! Now read the data from the files:

      dt_in = 1.0/sps_in
      t_in_fft = float(npw2)/sps_in
      df = 1.0/t_in_fft
      
      fnyq_in = 0.5*sps_in
      
      fupper_in_fft = real(npw2/2 - 1)*df   
      
      sps_out =  sps_in
      npts_out = npts_in


! Compute the Fourier spectra:

! Fill working array with input data:

      work = 0.0
 
      do i = 1, npts_in
        work(i) = ts_in(i)
      end do

!
!     remove dc, apply taper (to portion before added zeros)
!
      detrend = .false.
      call dcdt(work, dt, npts_in, 1, npts_in, 
     : dc_remove, detrend)

      data_length = dt_in * float(npts_in)
      frctnfront = taper_front/data_length
      frctnback =  taper_back/data_length
      call tprfrctn (frctnfront, frctnback, work, npts_in)
      
      isign = +1
      call realft(work, npw2, isign)

! The spectrum will be in work(1...npw2).   The real and complex values 
! are in adjacent elements of the array, except work(1) = real(F(0)), work(2) = real(F(N/2)) 
! (N/2 = Nyquist, since start indexing at 0).    To interpolate in the time domain,
! just inverse transform, since work has been prefilled with zeros beyond npw2_4input

! Apply a raised cosine taper from frequency_taper_factor*fnyq_in to fnyq_in, if desired:
      n1 = fcut1/df + 1              
      n2 = fcut2/df + 1                 
      pi = 4.0*atan(1.0)

!DEBUG
      print *,' sps_in, npw2, df, fcut1, fcut2, n1, n2, pi = ',
     :          sps_in, npw2, df, fcut1, fcut2, n1, n2, pi
!DEBUG
     
      do i = n1, n2 -1
        taper = (1.0/2.0**eta) * 
     :            (1.0+cos(pi*real(i - n1)/real(n2 - n1)))**eta
        work(2*i-1) = taper * work(2*i-1)   ! The real part of the FAS for frequency (i-1)*df
        work(2*i)   = taper * work(2*i)     ! The imaginary part of the FAS for frequency (i-1)*df
      end do
       
      do i = n2, npw2/2
        work(2*i-1) = 0.0
        work(2*i) = 0.0
      end do
       
      work(2) = 0.0   ! special case for the Nyquist frequency.


      isign = -1
      call realft(work, npw2, isign)
      
      sps_out = sps_in
      
! Store values in output array (and scale values)

      do i = 1, npts_out
        ts_out(i) = 2.0*work(i)/real(npw2)
      end do

      deallocate( work )   
 
      RETURN
      END
      
! -------------------------------------------------- hicut_fd_cosine_taper

      include '\forprogs\hicut_fd_cosine_taper_fft_subroutines.for'
        !This file is created
        !by calling make_hicut_fd_cosine_taper_fft_subroutines_file.bat
        !from within cl_smc_hicut_fd_cosine_taper.bat
      
!      include '\forprogs\get_npw2.for'
!      include '\forprogs\fork.for'
!      include '\forprogs\tprfrctn.for'
!      include '\forprogs\dcdt.for'
!      include '\forprogs\zeropad.for'
!      include '\forprogs\four1.for'
!      include '\forprogs\realft.for'
 

   
